/******************************************************************************\
 *
 * NHibernateEg.Tutorial1A
 * Copyright  2006, Pierre Henri Kuat. Tous droits rservs.
 *
 * Ce produit est sous les termes de la Licence Gnrale Publique Limite GNU.
 * Lire le fichier "licence.txt" pour plus de details.
 *
\*/

namespace NHibernateEg.Tutorial1A
{
	/// <summary>
	/// Boutique dans laquelle les commandes sont passes.
	/// </summary>
	public sealed class Boutique
	{
		private NHibernate.ISessionFactory _sessionFactory;


		/// <summary> Configuration de l'accs  la base de donnes. </summary>
		public Boutique(string baseDeDonnes, string chaneDeConnexion)
		{
			System.Console.Out.WriteLine("Configuration de NHibernate...\n");

			// Active l'inscription des informations de dbogages sur les oprations de NHibernate
			log4net.Config.XmlConfigurator.Configure();

			// Cre l'objet qui contiendras les informations de configuration
			// et assigne les informations pour accder  la base de donnes
			NHibernate.Cfg.Configuration cfg = new NHibernate.Cfg.Configuration();
			cfg.SetProperty(NHibernate.Cfg.Environment.ConnectionProvider, "NHibernate.Connection.DriverConnectionProvider");

			System.Console.Out.WriteLine("Utilise la base de donnes: <" + baseDeDonnes
				 + ">\nChaneDeConnexion: <" + chaneDeConnexion + ">\n");

			if("Access" == baseDeDonnes)
			{
				cfg.SetProperty(NHibernate.Cfg.Environment.Dialect, "NHibernate.JetDriver.JetDialect, NHibernate.JetDriver");
				cfg.SetProperty(NHibernate.Cfg.Environment.ConnectionDriver, "NHibernate.JetDriver.JetDriver, NHibernate.JetDriver");
			}
			else if("MSSQL" == baseDeDonnes)
			{
				cfg.SetProperty(NHibernate.Cfg.Environment.Dialect, "NHibernate.Dialect.MsSql2000Dialect");
				cfg.SetProperty(NHibernate.Cfg.Environment.ConnectionDriver, "NHibernate.Driver.SqlClientDriver");
			}
			else if("MySQL" == baseDeDonnes)
			{
				cfg.SetProperty(NHibernate.Cfg.Environment.Dialect, "NHibernate.Dialect.MySQLDialect");
				cfg.SetProperty(NHibernate.Cfg.Environment.ConnectionDriver, "NHibernate.Driver.MySqlDataDriver");
			}
			else
				throw new System.InvalidOperationException("La base de donnes '" + baseDeDonnes + "' est inconnue.");

			cfg.SetProperty(NHibernate.Cfg.Environment.ConnectionString, chaneDeConnexion);


			// Utilise NHibernate.Mapping.Attributes pour crer les informations sur les entits
			System.IO.MemoryStream flux = new System.IO.MemoryStream(); // Contenant des informations
			NHibernate.Mapping.Attributes.HbmSerializer.Default.Validate = true; // Active la validation (optionnel)
			// Demande  NHibernate d'utiliser les champs et non les proprits (dans les entits)
			NHibernate.Mapping.Attributes.HbmSerializer.Default.HbmDefaultAccess = "field.camelcase-underscore";
			// Rcupre les informations  partir de cette assemble (peut aussi tre fait classe par classe)
			System.Console.Out.WriteLine("NHibernate.Mapping.Attributes.HbmSerializer.Default.Serialize()...\n");
			NHibernate.Mapping.Attributes.HbmSerializer.Default.Serialize(flux, System.Reflection.Assembly.GetExecutingAssembly());
			flux.Position = 0;
			cfg.AddInputStream(flux); // Envoi les informations de Mappage  la Configuration de NHibernate
			flux.Close();


			// Cr le(s) table(s) dans la base de donnes pour les entits
			System.Console.Out.WriteLine("new NHibernate.Tool.hbm2ddl.SchemaExport(cfg).Create()...");
			new NHibernate.Tool.hbm2ddl.SchemaExport(cfg).Create(true, true);


			// Construit la "fabrique de sessions"
			System.Console.Out.WriteLine("\n\nsessionFact = cfg.BuildSessionFactory();\n\n");
			_sessionFactory = cfg.BuildSessionFactory();
		}




		/// <summary> Cration et enregistrement de 'n' commandes alatoires. </summary>
		public void GnreDesCommandesAlatoires(int n)
		{
			NHibernate.ISession session = null;
			NHibernate.ITransaction transaction = null;

			System.Console.Out.WriteLine("\nEnregistrement de " + n + " commandes alatoires...");
			try
			{
				session = _sessionFactory.OpenSession();
				transaction = session.BeginTransaction();

				for(int i=0; i<n; i++)
				{
					Commande c = new Commande();

					c.Produit = "P" + (i+1).ToString();
					c.Quantit = n - i;
					c.CalculeLePrixTotal(i * 10 + n);

					session.Save(c);
				}

				// Envoie les modifications (=> Gnre et excute les requtes)
				transaction.Commit();
			}
			catch
			{
				if(transaction != null)
					transaction.Rollback(); // Erreur => nous DEVONT annuler les modifications
				throw; // Ici, nous renvoyons la mme exception pour qu'elle soit gre (imprime)
			}
			finally
			{
				if(session != null)
					session.Close();
			}
		}


		/// <summary> Pour chaque commande (dans la base de donnes), impression de son identificateur, sa date et le nom du produit. </summary>
		public void AfficheToutesLesCommandes()
		{
			using(NHibernate.ISession session = _sessionFactory.OpenSession())
			{
				System.Collections.IList rsultat = session.Find("select c.Id, c.Date, c.Produit from Commande c");

				System.Console.Out.WriteLine("\n" + rsultat.Count + " commandes trouves!");
				foreach(System.Collections.IList l in rsultat)
					System.Console.Out.WriteLine("  Commande N"
						+ l[0] + ",  Date=" + ((System.DateTime)l[1]).ToString("u") + ",  Produit=" + l[2]);
			} // finally { session.Close(); }	est ralis par using()
		}


		/// <summary> Renvoi de la commande N 'id'. </summary>
		public Commande ChargeUneCommande(int id)
		{
			System.Console.Out.WriteLine("\nChargement de la commande N " + id + "...");
			using(NHibernate.ISession session = _sessionFactory.OpenSession())
				return session.Load(typeof(Commande), id) as Commande;
			// finally { session.Close(); }	est ralis par using()
		}


		/// <summary> Affichage de la commande sur la console. </summary>
		public void Affiche(Commande c)
		{
			System.Console.Out.WriteLine("\nCommande N"
				+ c.Id + "\n Date = " + c.Date.ToString("U") + "\n Produit=" + c.Produit
				+ ",  Quantit=" + c.Quantit + ",  PrixTotal=" + c.PrixTotal);
		}


		/// <summary> Insertion ou mise  jour de la commande (dans la base de donnes). </summary>
		public void Sauvegarde(Commande c)
		{
			NHibernate.ISession session = null;
			NHibernate.ITransaction transaction = null;

			// Voici comment la Session dcide d'insrer ou de mettre  jour; utilisez NHMA.Id(UnsavedValue=x) pour remplacer 0
			System.Console.Out.Write("\n"  +  (c.Id == 0  ?  "Insertion"  :  "Mise  jour"));
			System.Console.Out.WriteLine(" la commande N " + c.Id + "...");
			try
			{
				session = _sessionFactory.OpenSession();
				transaction = session.BeginTransaction();

				// La Session NHibernate vas automatiquement dtecter si elle doit effectuer un INSERT ou un UPDATE
				session.SaveOrUpdate(c);

				// Envoie les modifications (=> Gnre et excute les requtes)
				transaction.Commit();
			}
			catch
			{
				if(transaction != null)
					transaction.Rollback(); // Erreur => nous DEVONT annuler les modifications
				throw; // Ici, nous renvoyons la mme exception pour qu'elle soit gre (imprime)
			}
			finally
			{
				if(session != null)
					session.Close();
			}
		}


		/// <summary> Ajout de 'n' heures  la date de toutes les commandes. </summary>
		public void ChangeLeFuseauHoraire(int n)
		{
			NHibernate.ISession session = null;
			NHibernate.ITransaction transaction = null;

			System.Console.Out.WriteLine("\nChangement du fuseau horaire de toutes les commandes: n=" + n + "...");
			try
			{
				session = _sessionFactory.OpenSession();
				transaction = session.BeginTransaction();

				System.Collections.IList commandes = session.CreateCriteria(typeof(Commande)).List();//session.Find("from Commande");
				foreach(Commande c in commandes)
					c.ChangeLeFuseauHoraire(n);
				// Il est inutile d'appeler Update(), la Session vas automatiquement
				// dtecter les entits modifies (tant que c'est bien elle qui les as charg)
				System.Console.Out.WriteLine(commandes.Count + " commandes modifies!");

				// Envoie les modifications (=> Gnre et excute les requtes)
				transaction.Commit();
			}
			catch
			{
				if(transaction != null)
					transaction.Rollback(); // Erreur => nous DEVONT annuler les modifications
				throw; // Ici, nous renvoyons la mme exception pour qu'elle soit gre (imprime)
			}
			finally
			{
				if(session != null)
					session.Close();
			}
		}


		/// <summary> Effacement de la commande (dans la base de donnes). </summary>
		public void Dtruit(int id)
		{
			NHibernate.ISession session = null;
			NHibernate.ITransaction transaction = null;

			System.Console.Out.WriteLine("\nEffacement de la commande N " + id + "...");
			try
			{
				session = _sessionFactory.OpenSession();
				transaction = session.BeginTransaction();

				session.Delete("from Commande c where c.Id = :Id", id, NHibernate.NHibernateUtil.Int32);

				// Envoie les modifications (=> Gnre et excute les requtes)
				transaction.Commit();
			}
			catch
			{
				if(transaction != null)
					transaction.Rollback(); // Erreur => nous DEVONT annuler les modifications
				throw; // Ici, nous renvoyons la mme exception pour qu'elle soit gre (imprime)
			}
			finally
			{
				if(session != null)
					session.Close();
			}
		}
	}
}
