/******************************************************************************\
 *
 * NHibernateEg.Tutorial1A
 * Copyright  2006, Pierre Henri Kuat. Tous droits rservs.
 *
 * Ce produit est sous les termes de la Licence Gnrale Publique Limite GNU.
 * Lire le fichier "licence.txt" pour plus de details.
 *
\*/

namespace NHibernateEg.Tutorial1A
{
	/// <summary>
	/// Entit "Commande".
	/// Contient le nom du produit achet, la quantit livre et le prix total.
	/// </summary>
	[NHibernate.Mapping.Attributes.Class(Table="SimpleCommande")]
	public class Commande
	{
		private int _id = 0;
		private System.DateTime _date = System.DateTime.Now;
		private string _produit;
		private int _quantit;
		private int _prixTotal;

		/// <summary> Identificateur unique de cette commande. </summary>
		[NHibernate.Mapping.Attributes.Id(Name="Id")]
			[NHibernate.Mapping.Attributes.Generator(1, Class="native")]
		public virtual int Id
		{
			get { return _id; }
		}

		/// <summary> Date  laquelle la commande as t enregistre. </summary>
		[NHibernate.Mapping.Attributes.Property(Column="`Date`")]
		public virtual System.DateTime Date
		{
			get { return _date; }
		}

		/// <summary> Nom du produit command. </summary>
		[NHibernate.Mapping.Attributes.Property(NotNull=true)]
		public virtual string Produit
		{
			get { return _produit; }
			set { _produit = value; }
		}

		/// <summary> Nombre d'lments commands. </summary>
		[NHibernate.Mapping.Attributes.Property]
		public virtual int Quantit
		{
			get { return _quantit; }
			set
			{
				if(value <= 0)
					throw new System.ArgumentException("La quantit doit tre strictement positive!");
				_quantit = value;
			}
		}

		/// <summary> Prix total de la commande. </summary>
		[NHibernate.Mapping.Attributes.Property]
		public virtual int PrixTotal
		{
			get { return _prixTotal; }
			set { _prixTotal = value; }
		}



		/// <summary> Constructeur par dfaut. </summary>
		public Commande()
		{
		}


		/// <summary> Constructeur prenant toute les proprits. </summary>
		public Commande(string produit, int prixUnitaire, int quantit)
		{
			this.Produit = produit;
			this.Quantit = quantit; // Ne pas utiliser _quantit
			this.CalculeLePrixTotal(prixUnitaire);
		}


		/// <summary> Calcule le prix total de la commande en utilisant le prix unitaire (et la quantit). </summary>
		public void CalculeLePrixTotal(int prixUnitaire)
		{
			this.PrixTotal = prixUnitaire * this.Quantit;
		}


		/// <summary> Ajout de 'n' heures  la date. </summary>
		public virtual void ChangeLeFuseauHoraire(int n)
		{
			this._date = this.Date.AddHours(n);
		}
	}
}
